package com.agiledeveloper

import org.codehaus.groovy.transform.GroovyASTTransformation
import org.codehaus.groovy.control.CompilePhase
import org.codehaus.groovy.transform.ASTTransformation
import org.codehaus.groovy.ast.*
import org.codehaus.groovy.syntax.*
import org.codehaus.groovy.control.SourceUnit
import org.codehaus.groovy.ast.stmt.*
import org.codehaus.groovy.ast.expr.*
import org.codehaus.groovy.ast.builder.*

@GroovyASTTransformation(phase = CompilePhase.SEMANTIC_ANALYSIS)
class InjectCode implements ASTTransformation {
  void visit(ASTNode[] astNodes, SourceUnit sourceUnit) {
    def checkingAccountClassNode = astNodes[0].classes.find { it.name == 'CheckingAccount' }
    injectAuditMethod(checkingAccountClassNode)
  }
  
  void injectAuditMethod(checkingAccountClassNode) {
    def nonAuditMethods = 
      checkingAccountClassNode?.methods.findAll { it.name != 'audit' }
    
    nonAuditMethods?.each { injectMethodWithAudit(it) }
  }
  
  void injectMethodWithAudit(methodNode) {
    //  def callToAudit = new ExpressionStatement(
    //    new MethodCallExpression(
    //      new VariableExpression('this'),
    //      'audit',
    //      new ArgumentListExpression(methodNode.parameters)))
    
    //methodNode.code.statements.add(0, callToAudit)
    
    //alternately, we can use the ASTBuilder
    
    
    List<Statement> statements = new AstBuilder().buildFromCode { audit(amount) }
    def callToAudit = statements[0].statements[0].expression
    methodNode.code.statements.add(0, new ExpressionStatement(callToAudit))
  }
}